#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
    Set of IPython extensions used for teaching at ETS Montreal.
"""

__version__ = '0.0.10'

__all__ = [
    "mariadb",
    "mariadb_install",
    "web",
    "plantuml",
    "mongodb",
    "mongodb_install"
]
